import { Doodler, Vector } from "@bearmetal/doodler";
import { TrainCar } from "./train.ts";
import { getContextItem } from "../lib/context.ts";
import { ResourceManager } from "../lib/resources.ts";

export class LargeLady extends TrainCar {
  scale = 1;
  constructor() {
    const resources = getContextItem<ResourceManager>("resources");
    const img = resources.get<HTMLImageElement>("snr:sprite/LargeLady")!;
    super(50, 10, img, 160, 23, {
      at: new Vector(0, 0),
      width: 160,
      height: 23,
    });

    this.bogies = [
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 35 * this.scale,
        sprite: {
          at: new Vector(0, 23),
          width: 33,
          height: 19,
          offset: new Vector(-19, -9),
        },
      },
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 64 * this.scale,
        // sprite: {
        //   at: new Vector(0, 23),
        //   width: 33,
        //   height: 19,
        //   offset: new Vector(-19, -9.5),
        // },
        sprite: {
          at: new Vector(34, 23),
          width: 51,
          height: 19,
          offset: new Vector(-25.5, -9.5),
        },
      },
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 35 * this.scale,
        sprite: {
          at: new Vector(34, 23),
          width: 60,
          height: 19,
          offset: new Vector(-25.5, -9.5),
        },
        rotate: true,
      },
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 0,
        sprite: {
          at: new Vector(95, 23),
          width: 16,
          height: 19,
          offset: new Vector(-8, -9.5),
        },
      },
    ];
  }

  override draw(): void {
    const doodler = getContextItem<Doodler>("doodler");
    for (const b of this.bogies) {
      if (!b.sprite) continue;
      doodler.drawRotated(b.pos, b.angle + (b.rotate ? 0 : Math.PI), () => {
        doodler.drawSprite(
          this.img,
          b.sprite!.at,
          b.sprite!.width,
          b.sprite!.height,
          b.pos.copy().add(b.sprite!.offset ?? new Vector(0, 0)),
          b.sprite!.width,
          b.sprite!.height,
        );
      });
    }

    const b = this.bogies[2];
    const a = this.bogies[1];
    const origin = b.pos.copy().add(new Vector(18, 0).rotate(b.angle));
    // const origin = Vector.add(Vector.sub(a.pos, b.pos).div(2), b.pos);
    // const angle = Vector.sub(b.pos, a.pos).heading();

    // const difAngle = Vector.sub(b.pos, c.pos).heading();
    const angle = b.angle + Math.PI;
    // const avgAngle = (difAngle + angle) / 2;

    doodler.drawCircle(origin, 4, { color: "blue" });

    doodler.drawRotated(origin, angle, () => {
      this.sprite
        ? doodler.drawSprite(
          this.img,
          this.sprite.at,
          this.sprite.width,
          this.sprite.height,
          origin.copy().sub(
            this.imgWidth * this.scale / 2,
            this.imgHeight * this.scale / 2,
          ),
          this.imgWidth * this.scale,
          this.imgHeight * this.scale,
        )
        : doodler.drawImage(
          this.img,
          origin.copy().sub(this.imgWidth / 2, this.imgHeight / 2),
        );
    });
  }
}
